@@ -3,13 +3,14 @@ from __future__ import annotations
 from functools import partial
 from itertools import groupby
 from math import log2
-from typing import Callable, Iterable, Literal, Sequence, Type, cast, overload
+from typing import Callable, Iterable, Literal, Sequence, cast, overload
 
 from vsaa import Eedi3, Nnedi3, SuperSampler
-from vskernels import Catrom, Kernel, KernelT, Scaler, Spline144
+from vskernels import Catrom, Kernel, KernelT, Scaler, Spline144, ScalerT
 from vsmask.edge import EdgeDetect
 from vstools import (
-    check_variable, core, depth, get_depth, get_h, get_prop, get_w, get_y, join, normalize_seq, split, vs
+    check_variable, core, depth, get_depth, get_h, get_prop, get_w, get_y, join, normalize_seq, split, vs,
+    CustomValueError
 )
 
 from .helpers import scale_var_clip
@@ -82,9 +83,10 @@ def get_select_descale(
         }
 
         if len(group_by_kernel) < 2:
-            raise ValueError(
-                'get_select_descale: With KernelDiff mode you need to specify at least two kernels!\n'
-                '(First will be the main kernel, others will be compared to it)'
+            raise CustomValueError(
+                'With KernelDiff mode you need to specify at least two kernels!\n'
+                '(First will be the main kernel, others will be compared to it)',
+                get_select_descale
             )
 
         kernel_indices = {
@@ -129,12 +131,12 @@ def get_select_descale(
             return other_clip
 
     else:
-        raise ValueError('get_select_descale: incorrect descale mode specified!')
+        raise CustomValueError('Incorrect descale mode specified!', get_select_descale)
 
     return _select_descale, diff_clips
 
 
-DescKernelsT = Kernel | Type[Kernel] | str | Sequence[Kernel | Type[Kernel] | str]
+DescKernelsT = KernelT | Sequence[KernelT]
 
 
 @overload
@@ -257,15 +259,15 @@ def descale(
 
     norm_resolutions = list(zip(widths, heights))
     norm_kernels = [
-        kernel if isinstance(kernel, Kernel) else Kernel.from_param(kernel)()
+        kernel if isinstance(kernel, Kernel) else Kernel.ensure_obj(kernel, descale)
         for kernel in kernels
     ]
 
     if len(widths) != len(heights):
-        raise ValueError("descale: Number of heights and widths specified mismatch!")
+        raise CustomValueError('Number of heights and widths specified mismatch!', descale)
 
     if not norm_kernels:
-        raise ValueError("descale: You must specify at least one kernel!")
+        raise CustomValueError('You must specify at least one kernel!', descale)
 
     work_clip, *chroma = split(clip)
 
@@ -349,8 +351,7 @@ def descale(
 
 def mixed_rescale(
     clip: vs.VideoNode, width: None | int = None, height: int = 720,
-    kernel: KernelT = Catrom,
-    downscaler: Scaler | KernelT = SSIM,
+    kernel: KernelT = Catrom, downscaler: ScalerT = SSIM,
     credit_mask: vs.VideoNode | CreditMaskT | bool = partial(descale_detail_mask, thr=0.05, inflate=4, xxpand=(4, 2)),
     mix_strength: float = 0.25, show_mask: bool | int = False,
     # Default settings set to match insaneAA as closely as reasonably possible
@@ -391,15 +392,13 @@ def mixed_rescale(
     :param eedi3:           Eedi3 instance that will be used for supersampling.
 
     :return:                Rescaled clip with a downscaled clip merged with it and credits masked.
-
-    :raises ValueError:     ``mask_thr`` is not between 0.0–1.0.
     """
-    assert check_variable(clip, "mixed_rescale")
+    assert check_variable(clip, mixed_rescale)
 
     width = width or get_w(height, clip.width / clip.height, 1)
 
-    kernel = Kernel.ensure_obj(kernel)
-    downscaler = Kernel.ensure_obj(downscaler)
+    kernel = Kernel.ensure_obj(kernel, mixed_rescale)
+    downscaler = Kernel.ensure_obj(downscaler, mixed_rescale)
 
     bits = get_depth(clip)
     clip_y = get_y(clip)
