@@ -6,7 +6,7 @@ from typing import Any, Callable, Concatenate, Literal, cast
 
 from vsaa import Nnedi3
 from vsexprtools import ExprOp, combine, norm_expr
-from vskernels import Scaler
+from vskernels import Scaler, ScalerT
 from vsrgtools import LimitFilterMode, RepairMode, limit_filter, median_clips, repair, unsharp_masked
 from vstools import (
     EXPR_VARS, ColorRange, CustomIndexError, CustomOverflowError, P, check_ref_clip, inject_self, scale_8bit, vs
@@ -24,11 +24,11 @@ __all__ = [
 
 
 class MergeScalers(GenericScaler):
-    def __init__(self, *scalers: type[Scaler] | Scaler | tuple[type[Scaler] | Scaler, float | None]) -> None:
+    def __init__(self, *scalers: ScalerT | tuple[ScalerT, float | None]) -> None:
         if (l := len(scalers)) < 2:
             raise CustomIndexError(f'Not enough scalers passed! ({l})', self.__class__)
         elif len(scalers) > len(EXPR_VARS):
-            raise CustomIndexError(f'Too many scalers passed! ({l})')
+            raise CustomIndexError(f'Too many scalers passed! ({l})', self.__class__)
 
         if any(isinstance(s, tuple) for s in scalers):
             norm_scalers = [
@@ -63,7 +63,7 @@ class MergeScalers(GenericScaler):
         else:
             weight = 1.0 / len(scalers)
 
-            norm_scalers = [(scaler, weight) for scaler in scalers]
+            norm_scalers = [(scaler, weight) for scaler in scalers]  # type: ignore
 
         self.scalers = [
             (
@@ -95,13 +95,15 @@ class MergedFSRCNNX(GenericScaler):
     operator: Literal[ExprOp.MAX, ExprOp.MIN] | None = ExprOp.MIN
     masked: bool = True
 
-    reference: type[Scaler] | Scaler | vs.VideoNode = Nnedi3(0, opencl=None)
+    reference: ScalerT | vs.VideoNode = Nnedi3(0, opencl=None)
 
     range_out: ColorRange | None = None
 
     fsrcnnx_shader: FSRCNNXShaderT = FSRCNNXShader.x56
 
     def __post_init__(self) -> None:
+        super().__post_init__()
+
         if self.strength >= 100:
             raise CustomOverflowError('strength can\'t be more or equal to 100!', self.__class__)
         elif self.strength <= 0:
@@ -112,6 +114,8 @@ class MergedFSRCNNX(GenericScaler):
         if self.undershoot is None:
             self.undershoot = self.overshoot
 
+        self._fsrcnnx_shader = FSRCNNXShader.ensure_obj(self.fsrcnnx_shader, self.__class__)
+
     @inject_self
     def scale(  # type: ignore
         self, clip: vs.VideoNode, width: int, height: int, shift: tuple[float, float] = (0, 0),
@@ -119,15 +123,15 @@ class MergedFSRCNNX(GenericScaler):
     ) -> vs.VideoNode:
         assert (self.undershoot or self.undershoot == 0) and (self.overshoot or self.overshoot == 0)
 
-        fsrcnnx = self.fsrcnnx_shader.scale(clip, width, height, shift, **kwargs)
+        fsrcnnx = self._fsrcnnx_shader.scale(clip, width, height, shift, **kwargs)
 
         if isinstance(self.reference, vs.VideoNode):
             smooth = self.reference
 
             if shift != (0, 0):
-                smooth = self.kernel.shift(smooth, shift)
+                smooth = self._kernel.shift(smooth, shift)
         else:
-            smooth = self.reference.scale(clip, width, height, shift)
+            smooth = Scaler.ensure_obj(self.reference, self.__class__).scale(clip, width, height, shift)
 
         check_ref_clip(fsrcnnx, smooth)
 
@@ -177,7 +181,7 @@ class UnsharpedFSRCNNX(GenericScaler):
             Concatenate[vs.VideoNode, P], vs.VideoNode
         ] = partial(unsharp_masked, radius=2, strength=65),
         merge_mode: LimitFilterMode | bool = True,
-        reference: type[Scaler] | Scaler | vs.VideoNode = Nnedi3(0, opencl=None),
+        reference: ScalerT | vs.VideoNode = Nnedi3(0, opencl=None),
         fsrcnnx_shader: FSRCNNXShaderT = FSRCNNXShader.x56,
         *args: P.args, **kwargs: P.kwargs
     ) -> None:
@@ -186,7 +190,7 @@ class UnsharpedFSRCNNX(GenericScaler):
         self.merge_mode = merge_mode
 
         self.reference = reference
-        self.fsrcnnx_shader = fsrcnnx_shader
+        self.fsrcnnx_shader = Scaler.ensure_obj(fsrcnnx_shader, self.__class__)
 
         self.args = args
         self.kwargs = kwargs
@@ -202,9 +206,9 @@ class UnsharpedFSRCNNX(GenericScaler):
             smooth = self.reference
 
             if shift != (0, 0):
-                smooth = self.kernel.shift(smooth, shift)
+                smooth = self._kernel.shift(smooth, shift)
         else:
-            smooth = self.reference.scale(clip, width, height, shift)
+            smooth = Scaler.ensure_obj(self.reference, self.__class__).scale(clip, width, height, shift)
 
         check_ref_clip(fsrcnnx, smooth)
 
