@@ -6,11 +6,11 @@ from math import ceil
 from typing import Any
 
 from vsexprtools import aka_expr_available, expr_func
-from vskernels import Bilinear, Catrom, Kernel, KernelT, Scaler, SetsuCubic
+from vskernels import Bilinear, Catrom, Scaler, ScalerT, SetsuCubic
 from vsrgtools import box_blur, gauss_blur
 from vstools import (
     Matrix, MatrixT, PlanesT, Transfer, VSFunction, check_variable, core, depth, fallback, get_depth, get_w,
-    inject_self, vs
+    inject_self, vs, CustomValueError, check_ref_clip
 )
 
 from .gamma import gamma2linear, linear2gamma
@@ -26,8 +26,8 @@ __all__ = [
 @dataclass
 class DPID(Scaler):
     sigma: float = 0.1
-    ref: vs.VideoNode | Scaler | None = None
-    scaler: Scaler = Bilinear()
+    ref: vs.VideoNode | ScalerT | None = None
+    scaler: ScalerT = Bilinear
     planes: PlanesT = None
 
     @inject_self
@@ -35,15 +35,12 @@ class DPID(Scaler):
         self, clip: vs.VideoNode, width: int, height: int, shift: tuple[float, float] = (0, 0), **kwargs: Any
     ) -> vs.VideoNode:
         if isinstance(self.ref, vs.VideoNode):
-            assert clip.format and self.ref.format
-            if clip.format != self.ref.format:
-                raise ValueError('DPID.scale: ref clip must be the same format as clip!')
-
+            check_ref_clip(clip, self.ref)
             ref = self.ref
         else:
             ref = clip
 
-        scaler = self.ref if isinstance(self.ref, Scaler) else self.scaler
+        scaler = Scaler.ensure_obj(self.ref if isinstance(self.ref, Scaler) else self.scaler, self.__class__)
 
         if (ref.width, ref.height) != (width, height):
             ref = scaler.scale(ref, width, height)
@@ -62,7 +59,7 @@ class SSIM(Scaler):
     curve: Transfer | bool = False
     sigmoid: bool = False
     epsilon: float = 1e-6
-    scaler: Scaler = Catrom()
+    scaler: ScalerT = Catrom
 
     @inject_self
     def scale(  # type: ignore[override]
@@ -79,7 +76,7 @@ class SSIM(Scaler):
 def ssim_downsample(
     clip: vs.VideoNode, width: int | None = None, height: int = 720,
     smooth: float | VSFunction = ((3 ** 2 - 1) / 12) ** 0.5,
-    scaler: Scaler | KernelT = Catrom,
+    scaler: ScalerT = Catrom,
     curve: Transfer | bool = False, sigmoid: bool = False,
     shift: tuple[float, float] = (0, 0), epsilon: float = 1e-6
 ) -> vs.VideoNode:
@@ -122,11 +119,10 @@ def ssim_downsample(
     """
     assert clip.format
 
-    if not isinstance(scaler, Scaler):
-        scaler = Kernel.from_param(scaler)()
+    scaler = Scaler.ensure_obj(scaler, ssim_downsample)
 
     if isinstance(scaler, SSIM):
-        raise ValueError("SSIM: you can't have SSIM as a scaler for itself!")
+        raise CustomValueError("You can't have SSIM as a scaler for itself!", SSIM)
 
     if callable(smooth):
         filter_func = smooth
@@ -170,26 +166,29 @@ def ssim_downsample(
 
 @dataclass
 class DLISR(GenericScaler):
-    scaler: type[Scaler] | Scaler = DPID(0.5, SetsuCubic)
+    scaler: ScalerT = DPID(0.5, SetsuCubic)
     matrix: MatrixT | None = None
-
     device_id: int | None = None
 
+    def __post_init__(self) -> None:
+        super().__post_init__()
+
+        self._scaler = Scaler.ensure_obj(self.scaler, self.__class__)
+
     @inject_self
     def scale(  # type: ignore
         self, clip: vs.VideoNode, width: int, height: int, shift: tuple[float, float] = (0, 0),
         *, matrix: MatrixT | None = None, **kwargs: Any
     ) -> vs.VideoNode:
-        assert check_variable(clip, self.__class__)
-
         output = clip
-        matrix = None
+
+        assert check_variable(clip, self.__class__)
 
         if width > clip.width or height > clip.width:
             if not matrix:
-                matrix = self.matrix or Matrix.from_param(matrix, self.__class__) or Matrix.from_video(clip, False)
+                matrix = Matrix.from_param(matrix or self.matrix, self.__class__) or Matrix.from_video(clip, False)
 
-            output = self.kernel.resample(output, vs.RGBS, Matrix.RGB, matrix)
+            output = self._kernel.resample(output, vs.RGBS, Matrix.RGB, matrix)
             output = output.std.Limiter()
 
             max_scale = max(ceil(width / clip.width), ceil(height / clip.height))
@@ -197,13 +196,13 @@ class DLISR(GenericScaler):
             output = output.akarin.DLISR(max_scale, self.device_id)
 
         if (clip.width, clip.height) != (width, height):
-            output = self.scaler.scale(output, width, height, shift)
+            output = self._scaler.scale(output, width, height, shift)
         elif shift != (0, 0):
-            output = self.kernel.shift(output, shift)
+            output = self._kernel.shift(output, shift)
 
         assert check_variable(output, self.__class__)
 
         if output.format.id == clip.format.id:
             return output
 
-        return self.kernel.resample(output, clip, matrix)
+        return self._kernel.resample(output, clip, matrix)
